import yaml

from charmhelpers.core import host, hookenv
from charmhelpers.core.templating import render
from charms.reactive import (
    when, when_not, set_state, remove_state
)
from charms.reactive.helpers import any_file_changed, data_changed
from charms.layer import snap


SNAP_NAME = 'prometheus-snmp-exporter'
SVC_NAME = 'snap.prometheus-snmp-exporter.snmp-exporter'
PORT_DEF = 9116
SNMP_EXPORTER_YML_TMPL = 'snmp.yaml.j2'
CONF_FILE_PATH = '/var/snap/prometheus-snmp-exporter/current/snmp.yml'


def templates_changed(tmpl_list):
    return any_file_changed(['templates/{}'.format(x) for x in tmpl_list])


@when_not('snmp-exporter.installed')
def install_packages():
    hookenv.status_set('maintenance', 'Installing software')
    config = hookenv.config()
    channel = config.get('snap_channel', 'stable')
    # Work around https://pad.lv/1622782
    try:
        snap.install(SNAP_NAME, channel=channel, force_dangerous=False)
    except:
        pass
    set_state('snmp-exporter.installed')
    set_state('snmp-exporter.do-check-reconfig')


def validate_config(filename):
    return yaml.safe_load(open(filename))


@when('snmp-exporter.installed')
@when('snmp-exporter.do-reconfig-yaml')
def write_snmp_exporter_config_yaml():
    config = hookenv.config()
    render(source=SNMP_EXPORTER_YML_TMPL,
           target=CONF_FILE_PATH,
           context={'config': config}
           )
    validate_config(CONF_FILE_PATH)
    hookenv.open_port(PORT_DEF)
    set_state('snmp-exporter.do-restart')
    remove_state('snmp-exporter.do-reconfig-yaml')


@when('snmp-exporter.started')
def check_config():
    set_state('snmp-exporter.do-check-reconfig')


@when('snmp-exporter.do-check-reconfig')
def check_reconfig_snmp_exporter():
    config = hookenv.config()

    if data_changed('snmp-exporter.config', config):
        set_state('snmp-exporter.do-reconfig-yaml')

    if templates_changed([SNMP_EXPORTER_YML_TMPL]):
        set_state('snmp-exporter.do-reconfig-yaml')

    remove_state('snmp-exporter.do-check-reconfig')


@when('snmp-exporter.do-restart')
def restart_snmp_exporter():
    if not host.service_running(SVC_NAME):
        hookenv.log('Starting {}...'.format(SVC_NAME))
        host.service_start(SVC_NAME)
    else:
        hookenv.log('Restarting {}, config file changed...'.format(SVC_NAME))
        host.service_restart(SVC_NAME)
    hookenv.status_set('active', 'Ready')
    set_state('snmp-exporter.started')
    remove_state('snmp-exporter.do-restart')


# Relations
@when('snmp-exporter.started')
@when('snmp-exporter.available') # Relation name is "snmp-exporter"
def configure_snmp_exporter_relation(self, target):
    target.configure(PORT_DEF)
